@@ -3,9 +3,9 @@
  *
  * Get the canonicalized absolute pathnames. The basis for SAKURA and TOMOYO.
  *
- * Copyright (C) 2005-2008  NTT DATA CORPORATION
+ * Copyright (C) 2005-2009  NTT DATA CORPORATION
  *
- * Version: 1.6.4   2008/09/03
+ * Version: 1.6.7   2009/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -24,16 +24,17 @@
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 #include <linux/namei.h>
 #include <linux/mount.h>
-static const int lookup_flags = LOOKUP_FOLLOW;
+static const int ccs_lookup_flags = LOOKUP_FOLLOW;
 #else
-static const int lookup_flags = LOOKUP_FOLLOW | LOOKUP_POSITIVE;
+static const int ccs_lookup_flags = LOOKUP_FOLLOW | LOOKUP_POSITIVE;
 #endif
 #include <linux/realpath.h>
 #include <linux/proc_fs.h>
 #include <linux/ccs_common.h>
+#include <net/sock.h>
 
 /**
- * get_absolute_path - Get the path of a dentry but ignores chroot'ed root.
+ * ccs_get_absolute_path - Get the path of a dentry but ignores chroot'ed root.
  *
  * @dentry: Pointer to "struct dentry".
  * @vfsmnt: Pointer to "struct vfsmount".
@@ -50,8 +51,8 @@ static const int lookup_flags = LOOKUP_FOLLOW | LOOKUP_POSITIVE;
  * \ooo style octal string.
  * Character \ is converted to \\ string.
  */
-static int get_absolute_path(struct dentry *dentry, struct vfsmount *vfsmnt,
-			     char *buffer, int buflen)
+static int ccs_get_absolute_path(struct dentry *dentry, struct vfsmount *vfsmnt,
+				 char *buffer, int buflen)
 {
 	/***** CRITICAL SECTION START *****/
 	char *start = buffer;
@@ -96,13 +97,17 @@ static int get_absolute_path(struct dentry *dentry, struct vfsmount *vfsmnt,
 				char *ep;
 				const pid_t pid
 					= (pid_t) simple_strtoul(sp, &ep, 10);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
-				if (!*ep && pid == current->tgid) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+				const pid_t tgid
+					= task_tgid_nr_ns(current,
+							  dentry->d_sb->
+							  s_fs_info);
+				if (!*ep && pid == tgid && tgid) {
 					sp = "self";
 					cp = sp + 3;
 				}
 #else
-				if (!*ep && pid == current->pid) {
+				if (!*ep && pid == sys_getpid()) {
 					sp = "self";
 					cp = sp + 3;
 				}
@@ -178,6 +183,8 @@ static int get_absolute_path(struct dentry *dentry, struct vfsmount *vfsmnt,
 	/***** CRITICAL SECTION END *****/
 }
 
+#define SOCKFS_MAGIC 0x534F434B
+
 /**
  * ccs_realpath_from_dentry2 - Returns realpath(3) of the given dentry but ignores chroot'ed root.
  *
@@ -196,6 +203,20 @@ int ccs_realpath_from_dentry2(struct dentry *dentry, struct vfsmount *mnt,
 	struct vfsmount *d_mnt;
 	if (!dentry || !newname || newname_len <= 2048)
 		goto out;
+	/* Get better name for socket. */
+	if (dentry->d_sb && dentry->d_sb->s_magic == SOCKFS_MAGIC) {
+		struct inode *inode = dentry->d_inode;
+		struct socket *sock = inode ? SOCKET_I(inode) : NULL;
+		struct sock *sk = sock ? sock->sk : NULL;
+		if (sk) {
+			snprintf(newname, newname_len - 1,
+				 "socket:[family=%u:type=%u:protocol=%u]",
+				 sk->sk_family, sk->sk_type, sk->sk_protocol);
+		} else {
+			snprintf(newname, newname_len - 1, "socket:[unknown]");
+		}
+		return 0;
+	}
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 22)
 	if (dentry->d_op && dentry->d_op->d_dname) {
 		/* For "socket:[\$]" and "pipe:[\$]". */
@@ -241,7 +262,7 @@ int ccs_realpath_from_dentry2(struct dentry *dentry, struct vfsmount *mnt,
 	d_mnt = mntget(mnt);
 	/***** CRITICAL SECTION START *****/
 	ccs_realpath_lock();
-	error = get_absolute_path(d_dentry, d_mnt, newname, newname_len);
+	error = ccs_get_absolute_path(d_dentry, d_mnt, newname, newname_len);
 	ccs_realpath_unlock();
 	/***** CRITICAL SECTION END *****/
 	dput(d_dentry);
@@ -267,7 +288,7 @@ int ccs_realpath_from_dentry2(struct dentry *dentry, struct vfsmount *mnt,
  */
 char *ccs_realpath_from_dentry(struct dentry *dentry, struct vfsmount *mnt)
 {
-	char *buf = ccs_alloc(sizeof(struct ccs_page_buffer));
+	char *buf = ccs_alloc(CCS_MAX_PATHNAME_LEN, false);
 	if (buf && ccs_realpath_from_dentry2(dentry, mnt, buf,
 					     CCS_MAX_PATHNAME_LEN - 1) == 0)
 		return buf;
@@ -285,7 +306,7 @@ char *ccs_realpath_from_dentry(struct dentry *dentry, struct vfsmount *mnt)
 char *ccs_realpath(const char *pathname)
 {
 	struct nameidata nd;
-	if (pathname && path_lookup(pathname, lookup_flags, &nd) == 0) {
+	if (pathname && path_lookup(pathname, ccs_lookup_flags, &nd) == 0) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
 		char *buf = ccs_realpath_from_dentry(nd.path.dentry,
 						     nd.path.mnt);
@@ -300,32 +321,60 @@ char *ccs_realpath(const char *pathname)
 }
 
 /**
- * ccs_realpath_nofollow - Get realpath of a pathname.
+ * ccs_realpath_both - Get realpath of a pathname and symlink.
  *
  * @pathname: The pathname to solve.
+ * @ee:       Pointer to "struct ccs_execve_entry".
  *
- * Returns the realpath of @pathname on success, NULL otherwise.
+ * Returns 0 on success, negative value otherwise.
  */
-char *ccs_realpath_nofollow(const char *pathname)
+int ccs_realpath_both(const char *pathname, struct ccs_execve_entry *ee)
 {
 	struct nameidata nd;
-	if (pathname && path_lookup(pathname, lookup_flags ^ LOOKUP_FOLLOW,
-				    &nd) == 0) {
+	int ret;
+	bool is_symlink;
+	if (!pathname ||
+	    path_lookup(pathname, ccs_lookup_flags ^ LOOKUP_FOLLOW, &nd))
+		return -ENOENT;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-		char *buf = ccs_realpath_from_dentry(nd.path.dentry,
-						     nd.path.mnt);
-		path_put(&nd.path);
+	is_symlink = nd.path.dentry->d_inode &&
+		S_ISLNK(nd.path.dentry->d_inode->i_mode);
+	ret = ccs_realpath_from_dentry2(nd.path.dentry, nd.path.mnt,
+					ee->tmp, CCS_EXEC_TMPSIZE - 1);
+	path_put(&nd.path);
 #else
-		char *buf = ccs_realpath_from_dentry(nd.dentry, nd.mnt);
-		path_release(&nd);
+	is_symlink = nd.dentry->d_inode && S_ISLNK(nd.dentry->d_inode->i_mode);
+	ret = ccs_realpath_from_dentry2(nd.dentry, nd.mnt, ee->tmp,
+					CCS_EXEC_TMPSIZE - 1);
+	path_release(&nd);
 #endif
-		return buf;
+	if (ret)
+		return -ENOMEM;
+	if (strlen(ee->tmp) > CCS_MAX_PATHNAME_LEN - 1)
+		return -ENOMEM;
+	ee->program_path[CCS_MAX_PATHNAME_LEN - 1] = '\0';
+	if (!is_symlink) {
+		strncpy(ee->program_path, ee->tmp,
+			CCS_MAX_PATHNAME_LEN - 1);
+		return 0;
 	}
-	return NULL;
+	if (path_lookup(pathname, ccs_lookup_flags, &nd))
+		return -ENOENT;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+	ret = ccs_realpath_from_dentry2(nd.path.dentry, nd.path.mnt,
+					ee->program_path,
+					CCS_MAX_PATHNAME_LEN - 1);
+	path_put(&nd.path);
+#else
+	ret = ccs_realpath_from_dentry2(nd.dentry, nd.mnt, ee->program_path,
+					CCS_MAX_PATHNAME_LEN - 1);
+	path_release(&nd);
+#endif
+	return ret ? -ENOMEM : 0;
 }
 
 /**
- * round_up - Round up an integer so that the returned pointers are appropriately aligned.
+ * ccs_round_up - Round up an integer so that the returned pointers are appropriately aligned.
  *
  * @size: Size in bytes.
  *
@@ -334,7 +383,7 @@ char *ccs_realpath_nofollow(const char *pathname)
  * FIXME: Are there more requirements that is needed for assigning value
  * atomically?
  */
-static inline unsigned int round_up(const unsigned int size)
+static inline unsigned int ccs_round_up(const unsigned int size)
 {
 	if (sizeof(void *) >= sizeof(long))
 		return ((size + sizeof(void *) - 1)
@@ -344,8 +393,8 @@ static inline unsigned int round_up(const unsigned int size)
 			/ sizeof(long)) * sizeof(long);
 }
 
-static unsigned int allocated_memory_for_elements;
-static unsigned int quota_for_elements;
+static unsigned int ccs_allocated_memory_for_elements;
+static unsigned int ccs_quota_for_elements;
 
 /**
  * ccs_alloc_element - Allocate permanent memory for structures.
@@ -359,32 +408,31 @@ static unsigned int quota_for_elements;
 void *ccs_alloc_element(const unsigned int size)
 {
 	static DEFINE_MUTEX(lock);
-	static char *buf;
-	static unsigned int buf_used_len = PAGE_SIZE;
+	static char *ccs_buf;
+	static unsigned int ccs_buf_used_len = PAGE_SIZE;
 	char *ptr = NULL;
-	const unsigned int word_aligned_size = round_up(size);
+	const unsigned int word_aligned_size = ccs_round_up(size);
 	if (word_aligned_size > PAGE_SIZE)
 		return NULL;
 	mutex_lock(&lock);
-	if (buf_used_len + word_aligned_size > PAGE_SIZE) {
-		if (!quota_for_elements || allocated_memory_for_elements
-		    + PAGE_SIZE <= quota_for_elements)
+	if (ccs_buf_used_len + word_aligned_size > PAGE_SIZE) {
+		if (!ccs_quota_for_elements || ccs_allocated_memory_for_elements
+		    + PAGE_SIZE <= ccs_quota_for_elements)
 			ptr = kzalloc(PAGE_SIZE, GFP_KERNEL);
 		if (!ptr) {
 			printk(KERN_WARNING "ERROR: Out of memory "
 			       "for ccs_alloc_element().\n");
-			if (!sbin_init_started)
+			if (!ccs_policy_loaded)
 				panic("MAC Initialization failed.\n");
 		} else {
-			buf = ptr;
-			allocated_memory_for_elements += PAGE_SIZE;
-			buf_used_len = word_aligned_size;
-			ptr = buf;
+			ccs_buf = ptr;
+			ccs_allocated_memory_for_elements += PAGE_SIZE;
+			ccs_buf_used_len = word_aligned_size;
 		}
 	} else if (word_aligned_size) {
 		int i;
-		ptr = buf + buf_used_len;
-		buf_used_len += word_aligned_size;
+		ptr = ccs_buf + ccs_buf_used_len;
+		ccs_buf_used_len += word_aligned_size;
 		for (i = 0; i < word_aligned_size; i++) {
 			if (!ptr[i])
 				continue;
@@ -397,43 +445,43 @@ void *ccs_alloc_element(const unsigned int size)
 	return ptr;
 }
 
-static unsigned int allocated_memory_for_savename;
-static unsigned int quota_for_savename;
+static unsigned int ccs_allocated_memory_for_savename;
+static unsigned int ccs_quota_for_savename;
 
 #define MAX_HASH 256
 
 /* Structure for string data. */
-struct name_entry {
+struct ccs_name_entry {
 	struct list1_head list;
-	struct path_info entry;
+	struct ccs_path_info entry;
 };
 
 /* Structure for available memory region. */
-struct free_memory_block_list {
+struct ccs_free_memory_block_list {
 	struct list_head list;
 	char *ptr;             /* Pointer to a free area. */
 	int len;               /* Length of the area.     */
 };
 
-/* The list for "struct name_entry". */
-static struct list1_head name_list[MAX_HASH];
+/* The list for "struct ccs_name_entry". */
+static struct list1_head ccs_name_list[MAX_HASH];
 
 /**
  * ccs_save_name - Allocate permanent memory for string data.
  *
  * @name: The string to store into the permernent memory.
  *
- * Returns pointer to "struct path_info" on success, NULL otherwise.
+ * Returns pointer to "struct ccs_path_info" on success, NULL otherwise.
  *
  * The RAM is shared, so NEVER try to modify or kfree() the returned name.
  */
-const struct path_info *ccs_save_name(const char *name)
+const struct ccs_path_info *ccs_save_name(const char *name)
 {
-	static LIST_HEAD(fmb_list);
+	static LIST_HEAD(ccs_fmb_list);
 	static DEFINE_MUTEX(lock);
-	struct name_entry *ptr;
+	struct ccs_name_entry *ptr;
 	unsigned int hash;
-	struct free_memory_block_list *fmb;
+	struct ccs_free_memory_block_list *fmb;
 	int len;
 	char *cp;
 	if (!name)
@@ -446,16 +494,17 @@ const struct path_info *ccs_save_name(const char *name)
 	}
 	hash = full_name_hash((const unsigned char *) name, len - 1);
 	mutex_lock(&lock);
-	list1_for_each_entry(ptr, &name_list[hash % MAX_HASH], list) {
+	list1_for_each_entry(ptr, &ccs_name_list[hash % MAX_HASH], list) {
 		if (hash == ptr->entry.hash && !strcmp(name, ptr->entry.name))
 			goto out;
 	}
-	list_for_each_entry(fmb, &fmb_list, list) {
+	list_for_each_entry(fmb, &ccs_fmb_list, list) {
 		if (len <= fmb->len)
 			goto ready;
 	}
-	if (!quota_for_savename || allocated_memory_for_savename + PAGE_SIZE
-	    <= quota_for_savename)
+	if (!ccs_quota_for_savename ||
+	    ccs_allocated_memory_for_savename + PAGE_SIZE
+	    <= ccs_quota_for_savename)
 		cp = kzalloc(PAGE_SIZE, GFP_KERNEL);
 	else
 		cp = NULL;
@@ -465,13 +514,13 @@ const struct path_info *ccs_save_name(const char *name)
 		kfree(fmb);
 		printk(KERN_WARNING "ERROR: Out of memory "
 		       "for ccs_save_name().\n");
-		if (!sbin_init_started)
+		if (!ccs_policy_loaded)
 			panic("MAC Initialization failed.\n");
 		ptr = NULL;
 		goto out;
 	}
-	allocated_memory_for_savename += PAGE_SIZE;
-	list_add(&fmb->list, &fmb_list);
+	ccs_allocated_memory_for_savename += PAGE_SIZE;
+	list_add(&fmb->list, &ccs_fmb_list);
 	fmb->ptr = cp;
 	fmb->len = PAGE_SIZE;
  ready:
@@ -483,7 +532,7 @@ const struct path_info *ccs_save_name(const char *name)
 	ccs_fill_path_info(&ptr->entry);
 	fmb->ptr += len;
 	fmb->len -= len;
-	list1_add_tail_mb(&ptr->list, &name_list[hash % MAX_HASH]);
+	list1_add_tail_mb(&ptr->list, &ccs_name_list[hash % MAX_HASH]);
 	if (fmb->len == 0) {
 		list_del(&fmb->list);
 		kfree(fmb);
@@ -494,7 +543,7 @@ const struct path_info *ccs_save_name(const char *name)
 }
 
 /* Structure for temporarily allocated memory. */
-struct cache_entry {
+struct ccs_cache_entry {
 	struct list_head list;
 	void *ptr;
 	int size;
@@ -514,24 +563,53 @@ static kmem_cache_t *ccs_cachep;
 static int __init ccs_realpath_init(void)
 {
 	int i;
+	/* Constraint for ccs_save_name(). */
 	if (CCS_MAX_PATHNAME_LEN > PAGE_SIZE)
 		panic("Bad size.");
+	/* Constraint for "struct ccs_execve_entry"->tmp users. */
+	if (CCS_MAX_PATHNAME_LEN > CCS_EXEC_TMPSIZE)
+		panic("Bad size.");
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23)
-	ccs_cachep = kmem_cache_create("ccs_cache", sizeof(struct cache_entry),
+	ccs_cachep = kmem_cache_create("ccs_cache",
+				       sizeof(struct ccs_cache_entry),
 				       0, 0, NULL);
 #else
-	ccs_cachep = kmem_cache_create("ccs_cache", sizeof(struct cache_entry),
+	ccs_cachep = kmem_cache_create("ccs_cache",
+				       sizeof(struct ccs_cache_entry),
 				       0, 0, NULL, NULL);
 #endif
 	if (!ccs_cachep)
 		panic("Can't create cache.\n");
 	for (i = 0; i < MAX_HASH; i++)
-		INIT_LIST1_HEAD(&name_list[i]);
-	INIT_LIST1_HEAD(&KERNEL_DOMAIN.acl_info_list);
-	KERNEL_DOMAIN.domainname = ccs_save_name(ROOT_NAME);
-	list1_add_tail_mb(&KERNEL_DOMAIN.list, &domain_list);
-	if (ccs_find_domain(ROOT_NAME) != &KERNEL_DOMAIN)
-		panic("Can't register KERNEL_DOMAIN");
+		INIT_LIST1_HEAD(&ccs_name_list[i]);
+	INIT_LIST1_HEAD(&ccs_kernel_domain.acl_info_list);
+	ccs_kernel_domain.domainname = ccs_save_name(ROOT_NAME);
+	list1_add_tail_mb(&ccs_kernel_domain.list, &ccs_domain_list);
+	if (ccs_find_domain(ROOT_NAME) != &ccs_kernel_domain)
+		panic("Can't register ccs_kernel_domain");
+#ifdef CONFIG_TOMOYO_BUILTIN_INITIALIZERS
+	{
+		/* Load built-in policy. */
+		const int len = strlen(CONFIG_TOMOYO_BUILTIN_INITIALIZERS) + 1;
+		char *str = kmalloc(len, GFP_KERNEL);
+		if (!str)
+			panic("Out of memory.");
+		memmove(str, CONFIG_TOMOYO_BUILTIN_INITIALIZERS, len);
+		ccs_normalize_line(str);
+		if (*str) {
+			char *cp = str;
+			do {
+				char *cp2 = strchr(cp, ' ');
+				if (cp2)
+					*cp2++ = '\0';
+				ccs_write_domain_initializer_policy(cp, false,
+								    false);
+				cp = cp2;
+			} while (cp);
+		}
+		kfree(str);
+	}
+#endif
 	return 0;
 }
 
@@ -541,22 +619,23 @@ __initcall(ccs_realpath_init);
 core_initcall(ccs_realpath_init);
 #endif
 
-/* The list for "struct cache_entry". */
-static LIST_HEAD(cache_list);
+/* The list for "struct ccs_cache_entry". */
+static LIST_HEAD(ccs_audit_cache_list);
+static LIST_HEAD(ccs_acl_cache_list);
+static DEFINE_SPINLOCK(ccs_cache_list_lock);
 
-static DEFINE_SPINLOCK(cache_list_lock);
-
-static unsigned int dynamic_memory_size;
+static unsigned int ccs_dynamic_memory_size;
+static unsigned int ccs_quota_for_dynamic;
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
 /**
- * round2 - Rounded up to power-of-two value.
+ * ccs_round2 - Rounded up to power-of-two value.
  *
  * @size: Size in bytes.
  *
  * Returns power-of-two of @size.
  */
-static int round2(size_t size)
+static int ccs_round2(size_t size)
 {
 #if PAGE_SIZE == 4096
 	size_t bsize = 32;
@@ -570,15 +649,15 @@ static int round2(size_t size)
 #endif
 
 /**
- * ccs_alloc - Allocate memory for temporal purpose.
+ * ccs_alloc - Allocate memory for temporary purpose.
  *
  * @size: Size in bytes.
  *
  * Returns pointer to allocated memory on success, NULL otherwise.
  */
-void *ccs_alloc(const size_t size)
+void *ccs_alloc(const size_t size, const _Bool check_quota)
 {
-	struct cache_entry *new_entry;
+	struct ccs_cache_entry *new_entry;
 	void *ret = kzalloc(size, GFP_KERNEL);
 	if (!ret)
 		goto out;
@@ -593,14 +672,35 @@ void *ccs_alloc(const size_t size)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 	new_entry->size = ksize(ret);
 #else
-	new_entry->size = round2(size);
+	new_entry->size = ccs_round2(size);
 #endif
-	/***** CRITICAL SECTION START *****/
-	spin_lock(&cache_list_lock);
-	list_add_tail(&new_entry->list, &cache_list);
-	dynamic_memory_size += new_entry->size;
-	spin_unlock(&cache_list_lock);
-	/***** CRITICAL SECTION END *****/
+	if (check_quota) {
+		bool quota_exceeded = false;
+		/***** CRITICAL SECTION START *****/
+		spin_lock(&ccs_cache_list_lock);
+		if (!ccs_quota_for_dynamic ||
+		    ccs_dynamic_memory_size + new_entry->size
+		    <= ccs_quota_for_dynamic) {
+			list_add_tail(&new_entry->list, &ccs_audit_cache_list);
+			ccs_dynamic_memory_size += new_entry->size;
+		} else {
+			quota_exceeded = true;
+		}
+		spin_unlock(&ccs_cache_list_lock);
+		/***** CRITICAL SECTION END *****/
+		if (quota_exceeded) {
+			kfree(ret);
+			kmem_cache_free(ccs_cachep, new_entry);
+			ret = NULL;
+		}
+	} else {
+		/***** CRITICAL SECTION START *****/
+		spin_lock(&ccs_cache_list_lock);
+		list_add(&new_entry->list, &ccs_acl_cache_list);
+		ccs_dynamic_memory_size += new_entry->size;
+		spin_unlock(&ccs_cache_list_lock);
+		/***** CRITICAL SECTION END *****/
+	}
  out:
 	return ret;
 }
@@ -615,22 +715,30 @@ void *ccs_alloc(const size_t size)
 void ccs_free(const void *p)
 {
 	struct list_head *v;
-	struct cache_entry *entry = NULL;
+	struct ccs_cache_entry *entry = NULL;
 	if (!p)
 		return;
 	/***** CRITICAL SECTION START *****/
-	spin_lock(&cache_list_lock);
-	list_for_each(v, &cache_list) {
-		entry = list_entry(v, struct cache_entry, list);
-		if (entry->ptr != p) {
-			entry = NULL;
-			continue;
-		}
-		list_del(&entry->list);
-		dynamic_memory_size -= entry->size;
-		break;
+	spin_lock(&ccs_cache_list_lock);
+	list_for_each(v, &ccs_acl_cache_list) {
+		entry = list_entry(v, struct ccs_cache_entry, list);
+		if (entry->ptr == p)
+			break;
+		entry = NULL;
 	}
-	spin_unlock(&cache_list_lock);
+	if (!entry) {
+		list_for_each(v, &ccs_audit_cache_list) {
+			entry = list_entry(v, struct ccs_cache_entry, list);
+			if (entry->ptr == p)
+				break;
+			entry = NULL;
+		}
+	}
+	if (entry) {
+		list_del(&entry->list);
+		ccs_dynamic_memory_size -= entry->size;
+	}
+	spin_unlock(&ccs_cache_list_lock);
 	/***** CRITICAL SECTION END *****/
 	if (entry) {
 		kfree(p);
@@ -650,24 +758,29 @@ void ccs_free(const void *p)
 int ccs_read_memory_counter(struct ccs_io_buffer *head)
 {
 	if (!head->read_eof) {
-		const unsigned int shared = allocated_memory_for_savename;
-		const unsigned int private = allocated_memory_for_elements;
-		const unsigned int dynamic = dynamic_memory_size;
+		const unsigned int shared = ccs_allocated_memory_for_savename;
+		const unsigned int private = ccs_allocated_memory_for_elements;
+		const unsigned int dynamic = ccs_dynamic_memory_size;
 		char buffer[64];
 		memset(buffer, 0, sizeof(buffer));
-		if (quota_for_savename)
+		if (ccs_quota_for_savename)
 			snprintf(buffer, sizeof(buffer) - 1,
-				 "   (Quota: %10u)", quota_for_savename);
+				 "   (Quota: %10u)", ccs_quota_for_savename);
 		else
 			buffer[0] = '\0';
 		ccs_io_printf(head, "Shared:  %10u%s\n", shared, buffer);
-		if (quota_for_elements)
+		if (ccs_quota_for_elements)
 			snprintf(buffer, sizeof(buffer) - 1,
-				 "   (Quota: %10u)", quota_for_elements);
+				 "   (Quota: %10u)", ccs_quota_for_elements);
 		else
 			buffer[0] = '\0';
 		ccs_io_printf(head, "Private: %10u%s\n", private, buffer);
-		ccs_io_printf(head, "Dynamic: %10u\n", dynamic);
+		if (ccs_quota_for_dynamic)
+			snprintf(buffer, sizeof(buffer) - 1,
+				 "   (Quota: %10u)", ccs_quota_for_dynamic);
+		else
+			buffer[0] = '\0';
+		ccs_io_printf(head, "Dynamic: %10u%s\n", dynamic, buffer);
 		ccs_io_printf(head, "Total:   %10u\n",
 			      shared + private + dynamic);
 		head->read_eof = true;
@@ -687,8 +800,10 @@ int ccs_write_memory_quota(struct ccs_io_buffer *head)
 	char *data = head->write_buf;
 	unsigned int size;
 	if (sscanf(data, "Shared: %u", &size) == 1)
-		quota_for_savename = size;
+		ccs_quota_for_savename = size;
 	else if (sscanf(data, "Private: %u", &size) == 1)
-		quota_for_elements = size;
+		ccs_quota_for_elements = size;
+	else if (sscanf(data, "Dynamic: %u", &size) == 1)
+		ccs_quota_for_dynamic = size;
 	return 0;
 }
