@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2009  NTT DATA CORPORATION
  *
- * Version: 1.6.6   2009/02/02
+ * Version: 1.6.7   2009/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -16,31 +16,31 @@
 #include <linux/tomoyo.h>
 #include <linux/realpath.h>
 #include <linux/highmem.h>
-#include <linux/binfmts.h>
 
 /**
  * ccs_print_bprm - Print "struct linux_binprm" for auditing.
  *
  * @bprm: Pointer to "struct linux_binprm".
+ * @dump: Pointer to "struct ccs_page_dump".
  *
  * Returns the contents of @bprm on success, NULL otherwise.
  */
-static char *ccs_print_bprm(struct linux_binprm *bprm)
+static char *ccs_print_bprm(struct linux_binprm *bprm,
+			    struct ccs_page_dump *dump)
 {
-	static const int buffer_len = 4096 * 2;
-	char *buffer = ccs_alloc(buffer_len);
+	static const int ccs_buffer_len = 4096 * 2;
+	char *buffer = ccs_alloc(ccs_buffer_len, false);
 	char *cp;
 	char *last_start;
 	int len;
 	unsigned long pos = bprm->p;
-	int i = pos / PAGE_SIZE;
 	int offset = pos % PAGE_SIZE;
 	int argv_count = bprm->argc;
 	int envp_count = bprm->envc;
 	bool truncated = false;
 	if (!buffer)
 		return NULL;
-	len = snprintf(buffer, buffer_len - 1,
+	len = snprintf(buffer, ccs_buffer_len - 1,
 		       "argc=%d envc=%d argv[]={ ", argv_count, envp_count);
 	cp = buffer + len;
 	if (!argv_count) {
@@ -49,24 +49,16 @@ static char *ccs_print_bprm(struct linux_binprm *bprm)
 	}
 	last_start = cp;
 	while (argv_count || envp_count) {
-		struct page *page;
-		const char *kaddr;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23) && defined(CONFIG_MMU)
-		if (get_user_pages(current, bprm->mm, pos, 1, 0, 1, &page,
-				   NULL) <= 0)
+		if (!ccs_dump_page(bprm, pos, dump))
 			goto out;
 		pos += PAGE_SIZE - offset;
-#else
-		page = bprm->page[i];
-#endif
-		/* Map */
-		kaddr = kmap(page);
 		/* Read. */
 		while (offset < PAGE_SIZE) {
+			const char *kaddr = dump->data;
 			const unsigned char c = kaddr[offset++];
 			if (cp == last_start)
 				*cp++ = '"';
-			if (cp >= buffer + buffer_len - 32) {
+			if (cp >= buffer + ccs_buffer_len - 32) {
 				/* Reserve some room for "..." string. */
 				truncated = true;
 			} else if (c == '\\') {
@@ -110,46 +102,18 @@ static char *ccs_print_bprm(struct linux_binprm *bprm)
 			if (!argv_count && !envp_count)
 				break;
 		}
-		/* Unmap. */
-		kunmap(page);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23) && defined(CONFIG_MMU)
-		put_page(page);
-#endif
-		i++;
 		offset = 0;
 	}
 	*cp++ = '}';
 	*cp = '\0';
 	return buffer;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23) && defined(CONFIG_MMU)
  out:
-	snprintf(buffer, buffer_len - 1,
+	snprintf(buffer, ccs_buffer_len - 1,
 		 "argc=%d envc=%d argv[]={ ... } envp[]= { ... }",
 		 argv_count, envp_count);
 	return buffer;
-#endif
 }
 
-static DECLARE_WAIT_QUEUE_HEAD(grant_log_wait);
-static DECLARE_WAIT_QUEUE_HEAD(reject_log_wait);
-
-static DEFINE_SPINLOCK(audit_log_lock);
-
-/* Structure for audit log. */
-struct log_entry {
-	struct list_head list;
-	char *log;
-};
-
-/* The list for "struct log_entry". */
-static LIST_HEAD(grant_log);
-
-/* The list for "struct log_entry". */
-static LIST_HEAD(reject_log);
-
-static int grant_log_count;
-static int reject_log_count;
-
 /**
  * ccs_init_audit_log - Allocate buffer for audit logs.
  *
@@ -162,60 +126,134 @@ static int reject_log_count;
  */
 char *ccs_init_audit_log(int *len, struct ccs_request_info *r)
 {
-	static const char *mode_4[4] = {
+	static const char *ccs_mode_4[4] = {
 		"disabled", "learning", "permissive", "enforcing"
 	};
 	char *buf;
 	char *bprm_info = "";
 	struct timeval tv;
-	struct task_struct *task = current;
-	u32 tomoyo_flags = task->tomoyo_flags;
+	u32 ccs_flags = current->ccs_flags;
 	const char *domainname;
 	if (!r->domain)
-		r->domain = current->domain_info;
+		r->domain = ccs_current_domain();
 	domainname = r->domain->domainname->name;
 	do_gettimeofday(&tv);
 	*len += strlen(domainname) + 256;
-	if (r->bprm) {
-		bprm_info = ccs_print_bprm(r->bprm);
+	if (r->ee) {
+		bprm_info = ccs_print_bprm(r->ee->bprm, &r->ee->dump);
 		if (!bprm_info)
 			return NULL;
 		*len += strlen(bprm_info);
 	}
-	buf = ccs_alloc(*len);
+	buf = ccs_alloc(*len, true);
 	if (buf)
 		snprintf(buf, (*len) - 1,
 			 "#timestamp=%lu profile=%u mode=%s pid=%d uid=%d "
 			 "gid=%d euid=%d egid=%d suid=%d sgid=%d fsuid=%d "
 			 "fsgid=%d state[0]=%u state[1]=%u state[2]=%u %s\n"
 			 "%s\n",
-			 tv.tv_sec, r->profile, mode_4[r->mode],
+			 tv.tv_sec, r->profile, ccs_mode_4[r->mode],
 			 (pid_t) sys_getpid(), current_uid(), current_gid(),
 			 current_euid(), current_egid(), current_suid(),
 			 current_sgid(), current_fsuid(), current_fsgid(),
-			 (u8) (tomoyo_flags >> 24), (u8) (tomoyo_flags >> 16),
-			 (u8) (tomoyo_flags >> 8), bprm_info, domainname);
-	if (r->bprm)
+			 (u8) (ccs_flags >> 24), (u8) (ccs_flags >> 16),
+			 (u8) (ccs_flags >> 8), bprm_info, domainname);
+	if (r->ee)
 		ccs_free(bprm_info);
 	return buf;
 }
 
+/**
+ * ccs_update_task_state - Update task's state.
+ *
+ * @r:          Pointer to "struct ccs_request_info".
+ */
+static void ccs_update_task_state(struct ccs_request_info *r)
+{
+	/*
+	 * Don't change the lowest byte because it is reserved for
+	 * CCS_CHECK_READ_FOR_OPEN_EXEC / CCS_DONT_SLEEP_ON_ENFORCE_ERROR /
+	 * CCS_TASK_IS_EXECUTE_HANDLER / CCS_TASK_IS_POLICY_MANAGER.
+	 */
+	const struct ccs_condition_list *ptr = r->cond;
+	if (ptr) {
+		struct task_struct *task = current;
+		const u8 flags = ptr->post_state[3];
+		u32 ccs_flags = task->ccs_flags;
+		if (flags & 1) {
+			ccs_flags &= ~0xFF000000;
+			ccs_flags |= ptr->post_state[0] << 24;
+		}
+		if (flags & 2) {
+			ccs_flags &= ~0x00FF0000;
+			ccs_flags |= ptr->post_state[1] << 16;
+		}
+		if (flags & 4) {
+			ccs_flags &= ~0x0000FF00;
+			ccs_flags |= ptr->post_state[2] << 8;
+		}
+		task->ccs_flags = ccs_flags;
+		r->cond = NULL;
+	}
+}
+
+#ifndef CONFIG_TOMOYO_AUDIT
+
+/**
+ * ccs_write_audit_log - Write audit log.
+ *
+ * @is_granted: True if this is a granted log.
+ * @r:          Pointer to "struct ccs_request_info".
+ * @fmt:        The printf()'s format string, followed by parameters.
+ *
+ * Returns 0 on success, -ENOMEM otherwise.
+ */
+int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
+			const char *fmt, ...)
+{
+	ccs_update_task_state(r);
+	return 0;
+}
+
+#else
+
+static DECLARE_WAIT_QUEUE_HEAD(ccs_grant_log_wait);
+static DECLARE_WAIT_QUEUE_HEAD(ccs_reject_log_wait);
+
+static DEFINE_SPINLOCK(ccs_audit_log_lock);
+
+/* Structure for audit log. */
+struct ccs_log_entry {
+	struct list_head list;
+	char *log;
+};
+
+/* The list for "struct ccs_log_entry". */
+static LIST_HEAD(ccs_grant_log);
+
+/* The list for "struct ccs_log_entry". */
+static LIST_HEAD(ccs_reject_log);
+
+static int ccs_grant_log_count;
+static int ccs_reject_log_count;
+
 /**
  * ccs_can_save_audit_log - Check whether the kernel can save new audit log.
  *
- * @domain:     Pointer to "struct domain_info". NULL for current->domain_info.
+ * @domain:     Pointer to "struct ccs_domain_info".
+ *              NULL for ccs_current_domain().
  * @is_granted: True if this is a granted log.
  *
  * Returns true if the kernel can save, false otherwise.
  */
-static bool ccs_can_save_audit_log(const struct domain_info *domain,
+static bool ccs_can_save_audit_log(const struct ccs_domain_info *domain,
 				   const bool is_granted)
 {
 	if (is_granted)
-		return grant_log_count
-			< ccs_check_flags(domain, CCS_TOMOYO_MAX_GRANT_LOG);
-	return reject_log_count
-		< ccs_check_flags(domain, CCS_TOMOYO_MAX_REJECT_LOG);
+		return ccs_grant_log_count
+			< ccs_check_flags(domain, CCS_MAX_GRANT_LOG);
+	return ccs_reject_log_count
+		< ccs_check_flags(domain, CCS_MAX_REJECT_LOG);
 }
 
 /**
@@ -235,11 +273,9 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 	int pos;
 	int len;
 	char *buf;
-	struct log_entry *new_entry;
-	const struct condition_list *ptr;
-	struct task_struct *task = current;
+	struct ccs_log_entry *new_entry;
 	if (!r->domain)
-		r->domain = task->domain_info;
+		r->domain = ccs_current_domain();
 	if (!ccs_can_save_audit_log(r->domain, is_granted))
 		goto out;
 	va_start(args, fmt);
@@ -252,57 +288,32 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 	va_start(args, fmt);
 	vsnprintf(buf + pos, len - pos - 1, fmt, args);
 	va_end(args);
-	new_entry = ccs_alloc(sizeof(*new_entry));
+	new_entry = ccs_alloc(sizeof(*new_entry), true);
 	if (!new_entry) {
 		ccs_free(buf);
 		goto out;
 	}
 	new_entry->log = buf;
 	/***** CRITICAL SECTION START *****/
-	spin_lock(&audit_log_lock);
+	spin_lock(&ccs_audit_log_lock);
 	if (is_granted) {
-		list_add_tail(&new_entry->list, &grant_log);
-		grant_log_count++;
+		list_add_tail(&new_entry->list, &ccs_grant_log);
+		ccs_grant_log_count++;
 		ccs_update_counter(CCS_UPDATES_COUNTER_GRANT_LOG);
 	} else {
-		list_add_tail(&new_entry->list, &reject_log);
-		reject_log_count++;
+		list_add_tail(&new_entry->list, &ccs_reject_log);
+		ccs_reject_log_count++;
 		ccs_update_counter(CCS_UPDATES_COUNTER_REJECT_LOG);
 	}
-	spin_unlock(&audit_log_lock);
+	spin_unlock(&ccs_audit_log_lock);
 	/***** CRITICAL SECTION END *****/
 	if (is_granted)
-		wake_up(&grant_log_wait);
+		wake_up(&ccs_grant_log_wait);
 	else
-		wake_up(&reject_log_wait);
+		wake_up(&ccs_reject_log_wait);
 	error = 0;
  out:
-	/*
-	 * Update task's state.
-	 *
-	 * Don't change the lowest byte because it is reserved for
-	 * TOMOYO_CHECK_READ_FOR_OPEN_EXEC / CCS_DONT_SLEEP_ON_ENFORCE_ERROR /
-	 * TOMOYO_TASK_IS_EXECUTE_HANDLER.
-	 */
-	ptr = r->cond;
-	if (ptr) {
-		const u8 flags = ptr->post_state[3];
-		u32 tomoyo_flags = task->tomoyo_flags;
-		if (flags & 1) {
-			tomoyo_flags &= ~0xFF000000;
-			tomoyo_flags |= ptr->post_state[0] << 24;
-		}
-		if (flags & 2) {
-			tomoyo_flags &= ~0x00FF0000;
-			tomoyo_flags |= ptr->post_state[1] << 16;
-		}
-		if (flags & 4) {
-			tomoyo_flags &= ~0x0000FF00;
-			tomoyo_flags |= ptr->post_state[2] << 8;
-		}
-		task->tomoyo_flags = tomoyo_flags;
-		r->cond = NULL;
-	}
+	ccs_update_task_state(r);
 	return error;
 }
 
@@ -315,7 +326,7 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
  */
 int ccs_read_grant_log(struct ccs_io_buffer *head)
 {
-	struct log_entry *ptr = NULL;
+	struct ccs_log_entry *ptr = NULL;
 	if (head->read_avail)
 		return 0;
 	if (head->read_buf) {
@@ -324,13 +335,14 @@ int ccs_read_grant_log(struct ccs_io_buffer *head)
 		head->readbuf_size = 0;
 	}
 	/***** CRITICAL SECTION START *****/
-	spin_lock(&audit_log_lock);
-	if (!list_empty(&grant_log)) {
-		ptr = list_entry(grant_log.next, struct log_entry, list);
+	spin_lock(&ccs_audit_log_lock);
+	if (!list_empty(&ccs_grant_log)) {
+		ptr = list_entry(ccs_grant_log.next, struct ccs_log_entry,
+				 list);
 		list_del(&ptr->list);
-		grant_log_count--;
+		ccs_grant_log_count--;
 	}
-	spin_unlock(&audit_log_lock);
+	spin_unlock(&ccs_audit_log_lock);
 	/***** CRITICAL SECTION END *****/
 	if (ptr) {
 		head->read_buf = ptr->log;
@@ -351,10 +363,10 @@ int ccs_read_grant_log(struct ccs_io_buffer *head)
  */
 int ccs_poll_grant_log(struct file *file, poll_table *wait)
 {
-	if (grant_log_count)
+	if (ccs_grant_log_count)
 		return POLLIN | POLLRDNORM;
-	poll_wait(file, &grant_log_wait, wait);
-	if (grant_log_count)
+	poll_wait(file, &ccs_grant_log_wait, wait);
+	if (ccs_grant_log_count)
 		return POLLIN | POLLRDNORM;
 	return 0;
 }
@@ -368,7 +380,7 @@ int ccs_poll_grant_log(struct file *file, poll_table *wait)
  */
 int ccs_read_reject_log(struct ccs_io_buffer *head)
 {
-	struct log_entry *ptr = NULL;
+	struct ccs_log_entry *ptr = NULL;
 	if (head->read_avail)
 		return 0;
 	if (head->read_buf) {
@@ -377,13 +389,14 @@ int ccs_read_reject_log(struct ccs_io_buffer *head)
 		head->readbuf_size = 0;
 	}
 	/***** CRITICAL SECTION START *****/
-	spin_lock(&audit_log_lock);
-	if (!list_empty(&reject_log)) {
-		ptr = list_entry(reject_log.next, struct log_entry, list);
+	spin_lock(&ccs_audit_log_lock);
+	if (!list_empty(&ccs_reject_log)) {
+		ptr = list_entry(ccs_reject_log.next, struct ccs_log_entry,
+				 list);
 		list_del(&ptr->list);
-		reject_log_count--;
+		ccs_reject_log_count--;
 	}
-	spin_unlock(&audit_log_lock);
+	spin_unlock(&ccs_audit_log_lock);
 	/***** CRITICAL SECTION END *****/
 	if (ptr) {
 		head->read_buf = ptr->log;
@@ -404,10 +417,11 @@ int ccs_read_reject_log(struct ccs_io_buffer *head)
  */
 int ccs_poll_reject_log(struct file *file, poll_table *wait)
 {
-	if (reject_log_count)
+	if (ccs_reject_log_count)
 		return POLLIN | POLLRDNORM;
-	poll_wait(file, &reject_log_wait, wait);
-	if (reject_log_count)
+	poll_wait(file, &ccs_reject_log_wait, wait);
+	if (ccs_reject_log_count)
 		return POLLIN | POLLRDNORM;
 	return 0;
 }
+#endif
